package com.sun.jmx.mbeanserver;

import javax.management.openmbean.OpenDataException;
import java.io.InvalidObjectException;
import javax.management.openmbean.OpenType;
import java.lang.reflect.Type;

public abstract class MXBeanMapping
{
    private final Type javaType;
    private final OpenType<?> openType;
    private final Class<?> openClass;
    
    protected MXBeanMapping(Type javaType, OpenType<?> openType) {
        if (javaType == null || openType == null) {
            throw new NullPointerException("Null argument");
        }
        this.javaType = javaType;
        this.openType = openType;
        this.openClass = makeOpenClass(javaType, openType);
        return;
    }
    
    public final Type getJavaType() {
        return this.javaType;
    }
    
    public final OpenType<?> getOpenType() {
        return this.openType;
    }
    
    public final Class<?> getOpenClass() {
        return this.openClass;
    }
    
    private static Class<?> makeOpenClass(Type type, OpenType<?> openType) {
        if (type instanceof Class && ((Class)type).isPrimitive()) {
            return (Class)type;
        }
        try {
            return Class.forName(openType.getClassName(), false, MXBeanMapping.class.getClassLoader());
        }
        catch (ClassNotFoundException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    public abstract Object fromOpenValue(Object p0) throws InvalidObjectException;
    
    public abstract Object toOpenValue(Object p0) throws OpenDataException;
    
    public void checkReconstructible() throws InvalidObjectException {
    }
}
